@@ -5,7 +5,6 @@
 #include "gallery.h"
 
 #include "application.h"
-#include "config.h"
 #include "imagelist.h"
 #include "info.h"
 #include "loader.h"
@@ -14,6 +13,27 @@
 #include <stdlib.h>
 #include <string.h>
 
+// Configuration parameters
+#define CFG_SECTION    "gallery"
+#define CFG_SIZE       "size"
+#define CFG_SIZE_DEF   200
+#define CFG_CACHE      "cache"
+#define CFG_CACHE_DEF  100
+#define CFG_FILL       "fill"
+#define CFG_FILL_DEF   true
+#define CFG_WINDOW     "window"
+#define CFG_WINDOW_DEF ARGB(0, 0, 0, 0)
+#define CFG_BACKGR     "background"
+#define CFG_BACKGR_DEF ARGB(0xff, 0x20, 0x20, 0x20)
+#define CFG_SELECT     "select"
+#define CFG_SELECT_DEF ARGB(0xff, 0x40, 0x40, 0x40)
+#define CFG_BORDER     "border"
+#define CFG_BORDER_DEF ARGB(0xff, 0, 0, 0)
+#define CFG_SHADOW     "shadow"
+#define CFG_SHADOW_DEF ARGB(0xff, 0, 0, 0)
+#define CFG_AA         "antialiasing"
+#define CFG_AA_DEF     false
+
 // Scale for selected thumbnail
 #define THUMB_SELECTED_SCALE 1.15f
 
@@ -90,12 +110,36 @@ static void clear_thumbnails(void)
     ctx.thumbs = NULL;
 }
 
+/**
+ * Get thumbnail layout.
+ * @param cols,rows,gap layout description
+ */
+static void get_layout(size_t* cols, size_t* rows, size_t* gap)
+{
+    const size_t width = ui_get_width();
+    const size_t height = ui_get_height();
+    const size_t cnum = width / ctx.thumb_size;
+    const size_t gap_px = (width - (cnum * ctx.thumb_size)) / (cnum + 1);
+    const size_t rnum = (height - gap_px) / (ctx.thumb_size + gap_px);
+
+    if (cols) {
+        *cols = cnum;
+    }
+    if (rows) {
+        *rows = rnum;
+    }
+    if (gap) {
+        *gap = gap_px;
+    }
+}
+
 /** Reset loader queue. */
 static void reset_loader(void)
 {
     // get number of thumbnails on the screen
-    const size_t cols = ui_get_width() / ctx.thumb_size;
-    const size_t rows = ui_get_height() / ctx.thumb_size + 1;
+    size_t cols, rows;
+    get_layout(&cols, &rows, NULL);
+    ++rows;
     const size_t total = cols * rows;
 
     // search for nearest to selected
@@ -145,10 +189,11 @@ static void reset_loader(void)
 /** Update thumnails layout. */
 static void update_layout(void)
 {
-    const size_t cols = ui_get_width() / ctx.thumb_size;
-    const size_t rows = ui_get_height() / ctx.thumb_size;
+    size_t cols, rows;
     size_t distance;
 
+    get_layout(&cols, &rows, NULL);
+
     // if selection is not visible, put it on the center
     distance = image_list_distance(ctx.top, ctx.selected);
     if (distance > cols * rows) {
@@ -238,9 +283,12 @@ static bool skip_thumbnail(size_t index)
  */
 static void select_nearest(enum action_type direction)
 {
-    const size_t cols = ui_get_width() / ctx.thumb_size;
-    const size_t rows = ui_get_height() / ctx.thumb_size;
-    size_t index = ctx.selected;
+    size_t cols, rows;
+    size_t index;
+
+    get_layout(&cols, &rows, NULL);
+
+    index = ctx.selected;
 
     switch (direction) {
         case action_first_file:
@@ -293,9 +341,13 @@ static void select_nearest(enum action_type direction)
  */
 static void scroll_page(bool forward)
 {
-    const size_t distance =
-        ui_get_width() / ctx.thumb_size * ui_get_height() / ctx.thumb_size - 1;
-    const size_t selected = image_list_jump(ctx.selected, distance, forward);
+    size_t cols, rows;
+    size_t distance;
+    size_t selected;
+
+    get_layout(&cols, &rows, NULL);
+    distance = cols * rows - 1;
+    selected = image_list_jump(ctx.selected, distance, forward);
 
     if (selected != IMGLIST_INVALID && selected != ctx.selected) {
         const size_t top = image_list_jump(ctx.top, distance, forward);
@@ -344,13 +396,8 @@ static void draw_thumbnail(struct pixmap* window, ssize_t x, ssize_t y,
             const ssize_t thumb_h = thumb->height * THUMB_SELECTED_SCALE;
             const ssize_t tx = x + thumb_size / 2 - thumb_w / 2;
             const ssize_t ty = y + thumb_size / 2 - thumb_h / 2;
-            if (ctx.thumb_aa) {
-                pixmap_scale_bicubic(thumb, window, tx, ty,
-                                     THUMB_SELECTED_SCALE, image->alpha);
-            } else {
-                pixmap_scale_nearest(thumb, window, tx, ty,
-                                     THUMB_SELECTED_SCALE, image->alpha);
-            }
+            pixmap_scale(ctx.thumb_aa ? pixmap_bicubic : pixmap_nearest, thumb,
+                         window, tx, ty, THUMB_SELECTED_SCALE, image->alpha);
         }
 
         // shadow
@@ -390,22 +437,21 @@ static void draw_thumbnail(struct pixmap* window, ssize_t x, ssize_t y,
  */
 static void draw_thumbnails(struct pixmap* window)
 {
+    size_t cols, rows, gap;
     size_t index = ctx.top;
     ssize_t select_x = 0;
     ssize_t select_y = 0;
     const struct thumbnail* select_th = NULL;
 
     // thumbnails layout
-    const size_t cols = ui_get_width() / ctx.thumb_size;
-    const size_t rows = ui_get_height() / ctx.thumb_size + 1;
-    const size_t margin =
-        (ui_get_width() - (cols * ctx.thumb_size)) / (cols + 1);
+    get_layout(&cols, &rows, &gap);
+    ++rows;
 
     // draw
     for (size_t row = 0; row < rows; ++row) {
-        const ssize_t y = row * ctx.thumb_size + margin * (row + 1);
+        const ssize_t y = row * ctx.thumb_size + gap * (row + 1);
         for (size_t col = 0; col < cols; ++col) {
-            const ssize_t x = col * ctx.thumb_size + margin * (col + 1);
+            const ssize_t x = col * ctx.thumb_size + gap * (col + 1);
             const struct thumbnail* th = get_thumbnail(index);
 
             // draw preview, but postpone the selected item
@@ -529,79 +575,27 @@ static void on_image_load(struct image* image, size_t index)
     app_redraw();
 }
 
-/**
- * Custom section loader, see `config_loader` for details.
- */
-static enum config_status load_config(const char* key, const char* value)
+void gallery_init(struct config* cfg, struct image* image)
 {
-    enum config_status status = cfgst_invalid_value;
+    ctx.thumb_size =
+        config_get_num(cfg, CFG_SECTION, CFG_SIZE, 1, 1024, CFG_SIZE_DEF);
+    ctx.thumb_max =
+        config_get_num(cfg, CFG_SECTION, CFG_CACHE, 0, 1024, CFG_CACHE_DEF);
+    ctx.thumb_fill = config_get_bool(cfg, CFG_SECTION, CFG_FILL, CFG_FILL_DEF);
+    ctx.thumb_aa = config_get_bool(cfg, CFG_SECTION, CFG_AA, CFG_AA_DEF);
+    ctx.clr_window =
+        config_get_color(cfg, CFG_SECTION, CFG_WINDOW, CFG_WINDOW_DEF);
+    ctx.clr_background =
+        config_get_color(cfg, CFG_SECTION, CFG_BACKGR, CFG_BACKGR_DEF);
+    ctx.clr_select =
+        config_get_color(cfg, CFG_SECTION, CFG_SELECT, CFG_SELECT_DEF);
+    ctx.clr_border =
+        config_get_color(cfg, CFG_SECTION, CFG_BORDER, CFG_BORDER_DEF);
+    ctx.clr_shadow =
+        config_get_color(cfg, CFG_SECTION, CFG_SHADOW, CFG_SHADOW_DEF);
 
-    if (strcmp(key, "size") == 0) {
-        ssize_t num;
-        if (str_to_num(value, 0, &num, 0) && num >= 10 && num <= 1024) {
-            ctx.thumb_size = num;
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, "cache") == 0) {
-        ssize_t num;
-        if (str_to_num(value, 0, &num, 0) && num >= 0) {
-            ctx.thumb_max = num;
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, "fill") == 0) {
-        if (config_to_bool(value, &ctx.thumb_fill)) {
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, "window") == 0) {
-        if (config_to_color(value, &ctx.clr_window)) {
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, "background") == 0) {
-        if (config_to_color(value, &ctx.clr_background)) {
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, "select") == 0) {
-        if (config_to_color(value, &ctx.clr_select)) {
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, "border") == 0) {
-        if (config_to_color(value, &ctx.clr_border)) {
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, "shadow") == 0) {
-        if (config_to_color(value, &ctx.clr_shadow)) {
-            status = cfgst_ok;
-        }
-    } else if (strcmp(key, "antialiasing") == 0) {
-        if (config_to_bool(value, &ctx.thumb_aa)) {
-            status = cfgst_ok;
-        }
-    } else {
-        status = cfgst_invalid_key;
-    }
-
-    return status;
-}
-
-void gallery_create(void)
-{
-    ctx.thumb_size = 200;
-    ctx.thumb_max = 100;
-    ctx.thumb_fill = true;
-    ctx.top = IMGLIST_INVALID;
-    ctx.selected = IMGLIST_INVALID;
-    ctx.clr_background = 0xff202020;
-    ctx.clr_select = 0xff404040;
-    ctx.clr_border = 0xff000000;
-    ctx.clr_shadow = 0xff000000;
-
-    // register configuration loader
-    config_add_loader("gallery", load_config);
-}
-
-void gallery_init(struct image* image)
-{
     ctx.top = image_list_first();
+    ctx.selected = ctx.top;
     if (image) {
         add_thumbnail(image);
         select_thumbnail(image->index);
