@@ -77,9 +77,11 @@ void image_rotate(struct image* ctx, size_t angle);
  * Create thumbnail from full size image.
  * @param image original image
  * @param size thumbnail size in pixels
+ * @param fill thumbnail scale mode (fill/fit)
  * @param antialias use antialiasing
  */
-void image_thumbnail(struct image* image, size_t size, bool antialias);
+void image_thumbnail(struct image* image, size_t size, bool fill,
+                     bool antialias);
 
 /**
  * Set image format description.
