@@ -5,7 +5,7 @@
 #pragma once
 
 #include "action.h"
-#include "memdata.h"
+#include "config.h"
 
 #include <xkbcommon/xkbcommon.h>
 
@@ -19,6 +19,11 @@
 #define VKEY_SCROLL_DOWN  0x42000002
 #define VKEY_SCROLL_LEFT  0x42000003
 #define VKEY_SCROLL_RIGHT 0x42000004
+#define VKEY_MOUSE_LEFT   0x42000005
+#define VKEY_MOUSE_RIGHT  0x42000006
+#define VKEY_MOUSE_MIDDLE 0x42000007
+#define VKEY_MOUSE_SIDE   0x42000008
+#define VKEY_MOUSE_EXTRA  0x42000009
 
 /** Key binding list entry. */
 struct keybind {
@@ -30,9 +35,10 @@ struct keybind {
 };
 
 /**
- * Create global default key binding scheme.
+ * Initialize global default key binding scheme.
+ * @param cfg config instance
  */
-void keybind_create(void);
+void keybind_init(struct config* cfg);
 
 /**
  * Destroy global key binding scheme.
