@@ -26,11 +26,18 @@
 #define WINDOW_DEFAULT_WIDTH  800
 #define WINDOW_DEFAULT_HEIGHT 600
 
-// Mouse button
+// Mouse buttons, from <linux/input-event-codes.h>
 #ifndef BTN_LEFT
-#define BTN_LEFT 0x110 // from <linux/input-event-codes.h>
+#define BTN_LEFT   0x110
+#define BTN_RIGHT  0x111
+#define BTN_MIDDLE 0x112
+#define BTN_SIDE   0x113
+#define BTN_EXTRA  0x114
 #endif
 
+// Uncomment the following line to enable printing draw time
+// #define TRACE_DRAW_TIME
+
 /** UI context */
 struct ui {
     // wayland specific
@@ -61,6 +68,9 @@ struct ui {
         size_t width;
         size_t height;
         int32_t scale;
+#ifdef TRACE_DRAW_TIME
+        struct timespec draw_time;
+#endif
     } wnd;
 
     // cross-desktop
@@ -332,8 +342,38 @@ static void on_pointer_button(void* data, struct wl_pointer* wl_pointer,
                               uint32_t serial, uint32_t time, uint32_t button,
                               uint32_t state)
 {
+    const bool pressed = (state == WL_POINTER_BUTTON_STATE_PRESSED);
+
     if (button == BTN_LEFT) {
-        ctx.mouse.active = (state == WL_POINTER_BUTTON_STATE_PRESSED);
+        ctx.mouse.active = pressed;
+    }
+
+    if (pressed) {
+        xkb_keysym_t key;
+        switch (button) {
+            // TODO: Configurable drag
+            // case BTN_LEFT:
+            //     key = VKEY_MOUSE_LEFT;
+            //     break;
+            case BTN_RIGHT:
+                key = VKEY_MOUSE_RIGHT;
+                break;
+            case BTN_MIDDLE:
+                key = VKEY_MOUSE_MIDDLE;
+                break;
+            case BTN_SIDE:
+                key = VKEY_MOUSE_SIDE;
+                break;
+            case BTN_EXTRA:
+                key = VKEY_MOUSE_EXTRA;
+                break;
+            default:
+                key = XKB_KEY_NoSymbol;
+                break;
+        }
+        if (key != XKB_KEY_NoSymbol) {
+            app_on_keyboard(key, keybind_mods(ctx.xkb.state));
+        }
     }
 }
 
@@ -733,11 +773,23 @@ struct pixmap* ui_draw_begin(void)
 
     ctx.wnd.pm.data = wl_buffer_get_user_data(ctx.wnd.current);
 
+#ifdef TRACE_DRAW_TIME
+    clock_gettime(CLOCK_MONOTONIC, &ctx.wnd.draw_time);
+#endif
+
     return &ctx.wnd.pm;
 }
 
 void ui_draw_commit(void)
 {
+#ifdef TRACE_DRAW_TIME
+    struct timespec curr;
+    clock_gettime(CLOCK_MONOTONIC, &curr);
+    const double ns = (curr.tv_sec * 1000000000 + curr.tv_nsec) -
+        (ctx.wnd.draw_time.tv_sec * 1000000000 + ctx.wnd.draw_time.tv_nsec);
+    printf("Rendered in %.6f sec\n", ns / 1000000000);
+#endif
+
     wl_surface_attach(ctx.wl.surface, ctx.wnd.current, 0, 0);
     wl_surface_damage(ctx.wl.surface, 0, 0, ctx.wnd.width, ctx.wnd.height);
     wl_surface_set_buffer_scale(ctx.wl.surface, ctx.wnd.scale);
